"""``DuIvyTools`` package for analysis of GMX MD results files. Written by Charles Hahn.

Python modules
----------------
The package consists of the following Python modules:
* DIT : the CLI application.
* XPM : module to process XPM files.
* XVG : module to process XVG files.
* NDX : module to process NDX files.
* MDP : module to generate MDP files.
* HELP : module to provide help messages.

"""

